/** NoteLengths.qml
 * Shows the lengths of notes for track 0.
 * 
 * Written by Jotham Gates
 * Created 17/06/2024
 * Modified 17/06/2024
 */
import QtQuick 2.9
import MuseScore 3.0
import QtQuick.Controls 2.2
MuseScore { 
      menuPath: "Plugins.Display note lengths"
      description: "Displays note lengths of track 0"
      version: "0.0.0"
      pluginType: "dialog"
      requiresScore: true

      // Properties that can be changed
      width:  800
      height: 400

      // Global variables
      property var noteString: "";
      property var errorMsgs: "";
      property var isError: false;

      Component.onCompleted: {
            if (mscoreMajorVersion >= 4) {
                  title = "Displays note lengths";
                  // thumbnailName = "tuneplayer_logo.svg";
                  categoryCode = "export";
            }
      }

      onRun: {
            if (typeof curScore === 'undefined') {
                  // Exit (compatible with MS3 and MS4).
                  (typeof(quit) === 'undefined' ? Qt.quit : quit)()
            }
            runConverter();
      }

      function runConverter() {
            // Reset everything
            noteString = "MuseScore Studio V" + mscoreMajorVersion + "." + mscoreMinorVersion + "." + mscoreUpdateVersion + "\n[";
            errorMsgs = "Errors encounted:<br>";
            isError = false;
            
            // To make the play events be meaningful in MS4 hopefully.
            curScore.startCmd();
            curScore.createPlayEvents();
            curScore.endCmd();

            // Do the work
            applyToNotesInSelection();

            // Pretty and display
            noteString = noteString + ']';
            outputText.text = noteString;
            outputText.selectAll();

            if(isError) {
                  outputErrors.text = errorMsgs;
                  outputErrors.color = "red";
            } else {
                  outputErrors.text = "No errors encounterd";
                  outputErrors.color = "green";
            }
      }

      function applyToNotesInSelection() { // Originally based off one of the examples.
            var track = 0; // TODO: Selectable target track?
        
            // The getting elements from measures directly seems to only end up with a lot of clefs and no notes, so
            // this monstrosity is required instead that attempts to keep segments starting from the beginning and
            // measures in sync.
            var segment = curScore.firstSegment();
            var measure = curScore.firstMeasure;
            var lastNoteIsStop = false;
            while (segment) {
                  var inMeasure = true;
                  while (inMeasure) {
                        var element = segment.elementAt(track);
                        if (element) {
                              if (element.type === Element.CHORD) {
                                    var note = highestNote(element.notes);
                                    if(note) {
                                          //****************************************************************************
                                          // Export play event length
                                          //****************************************************************************
                                          noteString += note.playEvents[0].len + ", ";
                                    }
                              } else if (element.type === Element.REST) {
                                    //****************************************************************************
                                    // Export play event length
                                    //****************************************************************************
                                    noteString += "R" + note.playEvents[0].len + ", ";
                              }
                        }
                        inMeasure = segment.nextInMeasure != null;
                        segment = segment.next;
                  }
                  measure = measure.nextMeasure;
            }
      }
      // Finds the highest note in case there are multiple notes
      function highestNote(notes) {
            if(notes.length < 1) {
                  return null;
            }
            var highest = notes[0];
            for(var i = 1; i < notes.length; i++) {
                  if(notes[i].pitch > highest.pitch) {
                        highest = notes[i];
                  }
            }
            return highest;
      }
      
      Rectangle {
            id: leftSide
            color: "transparent"
            x: 180

            anchors.left: parent.left
            anchors.right: rightSide.left
            anchors.bottom: parent.bottom
            anchors.top: parent.top
            anchors.margins: 10

            Text {
                  id: optionsText
                  text: "Options"
            }

            Column {
                  anchors.top: optionsText.bottom;
                  anchors.topMargin: 5;
            }

            Rectangle {
                  id: copyButton
                  width: 170
                  height: 40
                  color: "#0040FF"
                  anchors.bottom: parent.bottom
                  anchors.left: parent.left
                  anchors.margins: 0
                  
                  Text {
                        text: "Copy to clipboard"
                        color: "white"
                        anchors.fill: parent
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                  }

                  MouseArea {
                        anchors.fill: parent
                        onClicked: {
                              outputText.selectAll();
                              outputText.copy();
                              outputErrors.text += "<br><i>Copied to clipboard</i>";
                        }
                  }
            }

      }
      Rectangle {
            id: rightSide
            color: "transparent"
            anchors.left: leftSide.right
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            anchors.top: parent.top
            anchors.margins: 10

            Text {
                  id: outputInfoText
                  text: "Output <i>(Copy and paste this into your sketch)</i>"
            }

            ScrollView { // Based off https://stackoverflow.com/a/53967239
                  id: outputBackground
                  anchors.left: parent.left
                  anchors.right: parent.right
                  anchors.bottom: rightDivider.top
                  anchors.top: outputInfoText.bottom
                  anchors.topMargin: 5
                  anchors.bottomMargin: 5
                  clip: true

                  Flickable {
                        contentHeight: Math.max(outputText.contentHeight + 100, parent.height);
                        //width: parent.width
                        contentWidth: Math.max(outputText.contentWidth + 20, parent.width);
                        Rectangle {
                              id : rectangle
                              color: "white"
                              anchors.fill: parent
                              TextEdit {
                                    id: outputText
                                    anchors.fill: parent
                                    anchors.margins: 10
                                    color: "black"
                                    font.family: "Courier";
                                    text: qsTr("Something went wrong... Maybe run from the plugin creator window.")
                              }
                        }
                  }

                  MouseArea {
                        anchors.fill: parent
                        onClicked: {
                              if(outputText.selectedText) {
                                    // Deselect
                                    outputText.deselect();
                              } else {
                                    outputText.selectAll();
                                    outputText.copy(); // Can't really get ctrl c to work, so will just use this
                                    outputErrors.text += "<br><i>Copied to clipboard</i>";
                              }
                        }
                  }
            }
            Text {
                  id: rightDivider
                  anchors.left: parent.left
                  anchors.right: parent.right
                  anchors.bottom: outputErrorsRect.top
                  anchors.bottomMargin: 5
                  text: "Messages"
            }

            ScrollView { // Based off https://stackoverflow.com/a/53967239
                  id: outputErrorsRect
                  anchors.left: parent.left
                  anchors.right: parent.right
                  anchors.bottom: parent.bottom
                  height: 60
                  clip: true

                  Flickable {
                        contentHeight: Math.max(outputErrors.contentHeight + 20, parent.height);
                        contentWidth: Math.max(outputErrors.contentWidth + 20, parent.width);
                        Rectangle {
                              color: "white"
                              anchors.fill: parent
                              Text {
                                    id: outputErrors
                                    anchors.fill: parent
                                    anchors.margins: 10
                                    color: "red"
                                    text: "Error messages go here."
                              }
                        }
                  }
            }
      }
}
